# coding=utf-8
from django_helpers.helpers.views import render_to_string
from django.forms import widgets
from django_helpers.templatetags.static_manger import add_js_file, require_jquery
from django_helpers.apps.forms.widgets import Widget

__author__ = 'ajumell'


class TinyMCEEditorOptions(object):
    pass


class TinyMCEWidget(Widget, widgets.Textarea):
    def __init__(self, *args, **kwargs):
        require_jquery()
        add_js_file('tiny_mce/tiny_mce.js')
        add_js_file('tiny_mce/jquery.tinymce.js')
        widgets.Textarea.__init__(self, *args, **kwargs)

    def render_js(self):
        return render_to_string('django-helpers/forms/tiny_mce.js', {
        })


# coding=utf-8
from django.core.urlresolvers import reverse
from django.template.defaultfilters import date
from . import DataTableColumn, get_value
from django_helpers.utils.bootstrap import link_button, generate_code, link_button_set


class BootstrapButtonColumn(DataTableColumn):
    def __init__(self, text, color, link, icon='', size=None, title_class=None, title_width="20px", title='', field='id'):
        DataTableColumn.__init__(self, field, title, False, False, title_class, title_width=title_width)
        self.text = text
        self.color = color
        self.link = link
        self.icon = icon
        self.size = size

    def get_html(self, request, instance):
        value = get_value(instance, self.field)
        try:
            link = reverse(self.link, args=(
                value,
            ))
        except:
            link = self.link

        return link_button(self.text, self.color, link, self.icon, self.size)


class BootstrapButtonSetColumn(DataTableColumn):
    def __init__(self, buttons, field='id', title='', title_class=None, title_width="20px", ):
        DataTableColumn.__init__(self, field, title, False, False, title_class, title_width)
        self.buttons = buttons

    def get_html(self, request, instance):
        data = []
        for button in self.buttons:
            text = button.get('text', '')
            color = button.get('color', '')
            link = button.get('link', '')
            icon = button.get('icon', '')
            size = button.get('size', '')

            field = button.get('field', self.field)
            value = getattr(instance, field)
            try:
                link = reverse(link, args=(value, ))
            except:
                pass
            data.append({
                'link': link,
                'text': text,
                'color': color,
                'icon': icon,
                'size': size
            })

        return link_button_set(data)


class LinkDataTableColumn(DataTableColumn):
    def __init__(self, text, link, title_width="20px", title='', field='id', title_class=None):
        DataTableColumn.__init__(self, field, title, False, False, title_class, title_width)
        self.text = text
        self.link = link

    def get_html(self, request, instance):
        value = getattr(instance, self.field)
        try:
            link = reverse(self.link, args=(
                value,
            ))
        except:
            link = self.link

        return generate_code('a', {
            'href': link
        }, self.text)


class EmailDataTableColumn(DataTableColumn):
    def __init__(self, field, title, searchable, sortable, title_class=None, title_width=None):
        DataTableColumn.__init__(self, field, title, searchable, sortable, title_class, title_width)

    def get_html(self, request, instance):
        value = DataTableColumn.get_html(self, request, instance)
        return generate_code('a', {
            'href': 'mailto:' + value
        }, value)


class DateDataTableColumn(DataTableColumn):
    def __init__(self, field, title, searchable, sortable, date_format=None, title_class=None, title_width=None):
        DataTableColumn.__init__(self, field, title, searchable, sortable, title_class, title_width)
        self.date_format = date_format

    def get_html(self, request, instance):
        value = DataTableColumn.get_html(self, request, instance)
        return date(value, self.date_format)



# coding=utf-8

from json import dumps
from hashlib import md5

from django.db.models import Q
from django.conf.urls import url
from django.http import HttpResponse

from django_helpers.templatetags.static_manger import require_jquery, require_jquery_ui


lookups = dict()

class AutoComplete(object):
    query_set = None
    search_fields = []
    url_search_parameters = None
    requires_auth = False
    id_name = 'id'
    label_name = 'label'

    def __init__(self):
        register(type(self))

    def get_queryset(self, request):
        """
        This function can be overridden to return custom query set.
        @return: The query set for performing extra searches.
        """
        return self.query_set

    def search_term(self, queryset, term):
        """
        Performs a simple database search for the term given.
        This function will performs a __contains in the given
        fields of the query set.

        @param term: str A search term
        @return: django.db.models.query.QuerySet a result query set.
        """
        query = None
        fields = self.search_fields
        for field in fields:
            kwargs = {field + "__contains": term}
            if query is None:
                query = Q(**kwargs)
            else:
                query = query | Q(**kwargs)
        return queryset.filter(query)

    def search_url_paramters(self, query, url_params):
        """
        This function performs a query lookup with the parameters from the url.
        @param query:
        @param url_params:
        @return:
        """
        url_search_parameters = self.url_search_parameters
        if url_search_parameters is not None:
            for parameter in url_search_parameters:
                args_dict = {parameter: url_params.get(parameter, '')}
                query = query.filter(**args_dict)
        return query

    def search(self, queryset, term, url_params):
        query = self.search_term(queryset, term)
        return self.search_url_paramters(query, url_params)

    def get_instances(self, ids):
        """
        Returns a queryset with the instances with given list of ids.
        This function is mainly used in many to many fields.

        @param ids: A list/enumerable of primary keys for which the datas has to be returned.
        @return: django.db.models.query.QuerySet
        """
        return self.query_set.filter(pk__in=ids)

    def get_instance(self, pk):
        """

        @param pk: The primary key for which the instance has to be returned.
        @return: A single instance of the current Model.
        """
        return self.query_set.get(pk=pk)

    def simple_format(self, instance, string, *fields):
        lst = []
        for term in fields:
            lst.append(getattr(instance, term))
        return string % tuple(lst)

    def format_value(self, instance):
        """
        Returns the formatted values that represents the current instance.
        @param instance:
        @return:
        """
        return unicode(instance)

    def get_search_term(self, request):
        """
        @author Muhammed K K
        @param request: django.http.HttpRequest current django request
        @return: str The term to be searched.
        """
        term = request.GET.get("term")
        if term is None:
            term = request.GET.get("q")
        return term

    def get_results(self, request, params):
        term = self.get_search_term(request)
        query = self.get_queryset(request)
        query = self.search(query, term, params)
        results = []
        for result in query:
            val = self.format_value(result)
            results.append({
                self.id_name: result.id,
                self.label_name: val,
            })
        return results


def autocomplete_lookup(request, lookup, **kwargs):
    """

    @param request: django.http.HttpRequest
    @param lookup: name of the lookup
    @param kwargs: extra url parameters
    @return: django.http.HttpResponse
    """
    # If lookup does not exists then return blank array.
    # This is prevent any exceptions at runtime.
    if not lookups.has_key(lookup):
        return HttpResponse("[]")

    auto_complete = lookups[lookup]()

    results = auto_complete.get_results(request, kwargs)
    return HttpResponse(dumps(results))


def create_reg(name):
    return "(?P<%s>.*)/" % name


url_prefix = 'auto-complete-'


def register(autocomplete_class):
    require_jquery()
    require_jquery_ui()

    if isinstance(autocomplete_class, AutoComplete):
        raise Exception("AutoComplete class is required not instance.")

    if not hasattr(autocomplete_class, 'name'):
        name = md5(unicode(autocomplete_class)).hexdigest()
    else:
        name = getattr(autocomplete_class, 'name')

    if not name.startswith(url_prefix):
        name = url_prefix + name

    setattr(autocomplete_class, 'name', name)

    if name in lookups:
        return

    lookups[name] = autocomplete_class
    reg = "%s/" % name
    extra_url_parameters = autocomplete_class.url_search_parameters
    if extra_url_parameters is not None:
        for parameter in extra_url_parameters:
            reg += create_reg(parameter)
    reg += "$"

    pattern = url(r"%s" % reg, autocomplete_lookup, name=name, kwargs={
        "lookup": name
    })
    urls.urlpatterns.append(pattern)
    return autocomplete_class

# Import last to disable recursive imports

from django_helpers import add_app_url
import urls
add_app_url('auto-complete', urls)


# coding=utf-8
from django import template
from django_helpers import get_settings_val
from django_helpers.helpers.templatetags import TemplateNode, parse_args
from django_helpers.apps.nivo_slider.models import NivoSlider as NivoSliderModel
from django_helpers.apps.nivo_slider import NivoSlider, use_theme
from django_helpers.templatetags.static_manger import add_js_file, add_css_file

__author__ = 'ajumell'

add_js_file('nivo-slider/js/jquery.nivo.slider.pack.js')
add_css_file('nivo-slider/css/nivo-slider.css')

for x in get_settings_val('NIVO_SLIDER_THEMES', []):
    use_theme(x)


class NivoSliderNode(TemplateNode):
    def render(self, context):
        name = self.args[0].resolve(context)
        slider_obj = NivoSliderModel.objects.get(name=name)
        slider = NivoSlider()
        for attr in dir(slider_obj):
            if not attr.startswith('__'):
                if hasattr(slider, attr):
                    val = getattr(slider_obj, attr)
                    setattr(slider, attr, val)
        slider.slider_id = slider_obj.id
        for img in slider_obj.slides():
            url = img.image.url
            caption = img.caption
            alt_text = img.alternate_text
            link = img.link
            transition = img.transition
            slider.add_image(url, caption, alt_text, link, transition)
        return unicode(slider)

register = template.Library()

@register.tag
def nivo_slider(parser, token):
    bits = token.split_contents()
    args, kwargs = parse_args(bits, parser)
    return NivoSliderNode(args, kwargs)